import java.util.Scanner;
import java.util.ArrayList;
import java.util.Collections;

/* This is a straightforward "shortest path" problem. 
 *
 * We construct a directed graph on the combatants using the history of battles, and 
 * then use Shortest Path algorithm to find the cost of victory of one vs the other. 
 *
 */
class MyDirectedGraph {
 	public final int numOfVertices;

 	private int[][] edgeWeights;

 	public MyDirectedGraph(int verticesIn) {
  		numOfVertices = verticesIn;
  		edgeWeights = new int[numOfVertices][numOfVertices];
		for (int i=0; i<numOfVertices; i++) {
			for (int j=0; j<numOfVertices; j++) {
				edgeWeights[i][j] = -1;
			}
		}
 	}

 	public void addEdge(int from, int to, int value) {
  		edgeWeights[from][to] = value;
 	}

 	public int costOfEdge(int from, int to) {
  		return edgeWeights[from][to];
 	}

    /* Find shortest paths using the Djkstra's algorithm. */
 	public int shortestPath(int from, int to) {
		int[] pred = new int[numOfVertices];
		for (int i=0; i<numOfVertices; i++) {
			pred[i] = -1;
		}
		int[] cost = new int[numOfVertices];
		for (int i=0; i<numOfVertices; i++) {
			cost[i] = -1;
		}

		cost[from] = 0;

		ArrayList<Integer> queue = new ArrayList<Integer>();
		for (int i=0; i<numOfVertices; i++) {
			queue.add(i);
		}

		int u, posOfMin, valOfMin;
		while (!queue.isEmpty()) {
			//remove u with lowest cost from start
			u = 0;
			posOfMin = 0;
			valOfMin = -1;
			for (int i=0; i<queue.size(); i++) {
				int candidateU = queue.get(i);
				if ( (cost[candidateU] != -1) && ((valOfMin == -1) || (cost[candidateU] < valOfMin)) ) {
					u = candidateU;
					valOfMin = cost[candidateU];
					posOfMin = i;
				}
			}
			queue.remove(posOfMin);
			//remove u with lowest cost from start
	
			if (valOfMin != -1) {
				for (int v=0; v<numOfVertices; v++) {
					if (costOfEdge(u,v)!=-1) {
						if (cost[v] == -1  ||  cost[v] > cost[u]+costOfEdge(u,v)) {
							cost[v] = cost[u]+costOfEdge(u,v);
							pred[v] = u;
						}
					}
				}
			}
		}

		return cost[to];
 	}
}


class SingleBattle {
    String combatant1;
    String combatant2;
    int costOfVictory;

    SingleBattle(String combatant1, String combatant2, int costOfVictory) {
        this.combatant1 = combatant1;
        this.combatant2 = combatant2;
        this.costOfVictory = costOfVictory;
    }
}

public class Battle {

    private static int solveBattle(int numCombatants, ArrayList<SingleBattle> pastBattles, String combatant1, String combatant2) {
        ArrayList<String> listOfNames = new ArrayList<String>();

        MyDirectedGraph battleGraph = new MyDirectedGraph(numCombatants);

        int combatant1pos;
        int combatant2pos;

        for(SingleBattle sb : pastBattles) {
            combatant1pos = listOfNames.indexOf(sb.combatant1);
            if (combatant1pos == -1) {
                listOfNames.add(sb.combatant1);
                combatant1pos = listOfNames.indexOf(sb.combatant1);
            }
            combatant2pos = listOfNames.indexOf(sb.combatant2);
            if (combatant2pos == -1) {
                listOfNames.add(sb.combatant2);
                combatant2pos = listOfNames.indexOf(sb.combatant2);
            }

            int currCost = battleGraph.costOfEdge(combatant1pos, combatant2pos);
            if ( (currCost == -1) || (currCost > sb.costOfVictory) ) {
                battleGraph.addEdge(combatant1pos, combatant2pos, sb.costOfVictory);
            }
        }

        combatant1pos = listOfNames.indexOf(combatant1);

        combatant2pos = listOfNames.indexOf(combatant2);

        if((combatant1pos == -1) || (combatant2pos == -1)) {
            return 0;
        }

        //Find out if one beats the other and at what cost
        int cost1beats2 = battleGraph.shortestPath(combatant1pos, combatant2pos);
        int cost2beats1 = battleGraph.shortestPath(combatant2pos, combatant1pos);

        //System.out.println(cost1beats2 + " " + cost2beats1);

        //Output appropriate message
        if (cost1beats2 == cost2beats1) {
            return 0;
        }
        else if (cost2beats1 == -1 || cost1beats2 < cost2beats1) {
            return 1;
        }
        else {
            return 2;
        }
    }

    public static void main(String[] args) {
        Scanner sc = new Scanner(System.in);

        int numCases = sc.nextInt();

        for(int ii = 0; ii < numCases; ii++) {

            int numCombatants = sc.nextInt();

            int numOfBattles = sc.nextInt();

            ArrayList<SingleBattle> pastBattles = new ArrayList<SingleBattle>();

            for (int i=0; i<numOfBattles; i++)
                pastBattles.add(new SingleBattle(sc.next(), sc.next(), sc.nextInt()));

            /* Read the new battle participants, and call subroutine. */
            String combatant1 = sc.next();
            String combatant2 = sc.next();
            int ret = solveBattle(numCombatants, pastBattles, combatant1, combatant2);

            if (ret == 0) 
                System.out.println("FIGHT!");
            else {
                if (ret == 1)
                    System.out.println(combatant1);
                else
                    System.out.println(combatant2);
            }
        }
    }
}



